/*******************************************************************************
 * Copyright (c) 2025, Jan Koester jan.koester@gmx.net
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 * Neither the name of the <organization> nor the
 *      names of its contributors may be used to endorse or promote products
 *      derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL <COPYRIGHT HOLDER> BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *******************************************************************************/

#include <string>
#include <vector>
#include <cstring>

#include <uuid/uuid.h>

#include "backend.h"
#include "authdb.h"

#pragma once

namespace authdb {
    class UserData {
    public:
        UserData(uuid_t id);
        ~UserData();

        void setUserName(const char* src);
        void getUsername(std::string &dest);

        void setFirstName(const char* src);
        void getFirstname(std::string &dest);

        void setLastName(const char* src);
        void getLastename(std::string &dest);

        void setMail(const char* src);
        void getMail(std::string &dest);

        void setPwHash(const char* src);
        void getPwHash(std::string &dest);

        void setAvatar(const std::vector<char> &src);
        void getAvatar(std::vector<char> &dest);

    protected:
        AuthDataRecord  *Data;
    private:
        AuthDataRecord  *username;
        AuthDataRecord  *firstname;
        AuthDataRecord  *lastname;
        AuthDataRecord  *mail;
        AuthDataRecord  *pwhash;
        //optional
        AuthDataRecord  *avatar;
        char             uid[255];
        friend class User;
    };

    class User {
    public:
        void create(AuthBackend &backend,UserData *dat);
        void remove(AuthBackend &backend,uuid_t uid);

        void edit(AuthBackend &backend,uuid_t uid,UserData &dat,size_t pos);
        void info(AuthBackend &backend,uuid_t uid,UserData &dat,size_t pos);

        void getAdditional(AuthBackend &backend,uuid_t uid,struct AuthData &data,size_t pos);
        void remAdditional(AuthBackend &backend,uuid_t uid,const char *fieldname);
        void editAdditional(AuthBackend &backend,uuid_t uid,struct AuthData &data,const char *fieldname);
    };

    class UserAPi{
    public:
        UserAPi();
        ~UserAPi();
    private:
    };

};
